@@ -1,4 +1,4 @@
-/* Copyright (C) 2002-2008 Stefan Buehler  <sbuehler@ltu.se>
+/* Copyright (C) 2002-2012 Stefan Buehler  <sbuehler@ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -34,6 +34,7 @@
 #include "absorption.h"
 #include "auto_md.h"
 #include "abs_species_tags.h"
+#include "global_data.h"
 
 //! Constructor from a tag definition String. 
 /*! 
@@ -46,18 +47,33 @@
 */
 SpeciesTag::SpeciesTag(String def) 
 {
+  // Save input string for error messages:
+  String def_original = def;
+    
   // Species lookup data:
-  extern const Array<SpeciesRecord> species_data;
-  // Name of species and isotope (aux variables):
+  using global_data::species_data;
+  // Name of species and isotopologue (aux variables):
   String name, isoname;
   // Aux index:
   Index n;
 
+  // Set default values for isotopologue
+  misotopologue = -1;
+
   // Set frequency limits to default values (no limits):
   mlf = -1;
   muf = -1;
 
-  // We cannot set a default value for the isotope, because the
+  // Set CIA species to -1 by default
+  mcia_second = -1;
+
+  // Set type to normal LBL species by default
+  mtype = TYPE_PLAIN;
+  
+  // Set line mixing type to none by default
+  mline_mixing_type = LINE_MIXING_TYPE_NONE;
+
+  // We cannot set a default value for the isotopologue, because the
   // default should be `ALL' and the value for `ALL' depends on the
   // species. 
     
@@ -82,6 +98,25 @@ SpeciesTag::SpeciesTag(String def)
   // (This will also remove possible whitespace.)
   mspecies = species_index_from_species_name( name );
 
+  // Remove whitespace
+  name.trim();
+
+  // Check if species name contains the special tag for
+  // Faraday Rotation
+  if (name == "free_electrons")
+    {
+      mtype = TYPE_FREE_ELECTRONS;
+      return;
+    }
+
+  // Check if species name contains the special tag for
+  // Particles
+  if (name == "particles")
+    {
+      mtype = TYPE_PARTICLES;
+      return;
+    }
+
   if ( 0 > mspecies )
     {
       ostringstream os;
@@ -95,61 +130,141 @@ SpeciesTag::SpeciesTag(String def)
   if ( 0 == def.nelem() )
     {
       // This means that there is nothing else to parse. Apparently
-      // the user wants all isotopes and no frequency limits.
-      // Frequency defaults are already set. Set isotope defaults:
-      misotope = spr.Isotope().nelem();
-      // This means all isotopes.
+      // the user wants all isotopologues and no frequency limits.
+      // Frequency defaults are already set. Set isotopologue defaults:
+      misotopologue = spr.Isotopologue().nelem();
+      // This means all isotopologues.
 
       return;
     }
     
-  // Extract the isotope name:
+  // Extract the isotopologue name/Zeeman flag:
   n    = def.find('-');    // find the '-'
   if (n != def.npos )
     {
-      isoname = def.substr(0,n);          // Extract before '-'
+      isoname = def.substr(0,n);    // Extract before '-'
       def.erase(0,n+1);             // Remove from def
+
+      if ("Z" == isoname)
+        {
+          mtype = TYPE_ZEEMAN;
+          // Zeeman flag was present, now extract the isotopologue name:
+          n    = def.find('-');    // find the '-'
+          if (n != def.npos )
+            {
+              isoname = def.substr(0,n);    // Extract before '-'
+              def.erase(0,n+1);             // Remove from def
+            }
+          else
+            {
+              // n==def.npos means that def does not contain a '-'. In that
+              // case we assume that it contains just the isotopologue name and
+              // nothing else.
+              isoname = def;
+              def  = "";
+            }
+        }
     }
   else
     {
       // n==def.npos means that def does not contain a '-'. In that
-      // case we assume that it contains just the isotope name and
-      // nothing else.
+      // case we assume that it contains just the isotopologue name or
+      // Zeeman flag and nothing else.
       isoname = def;
       def  = "";
+      if ("Z" == isoname)
+        {
+          mtype = TYPE_ZEEMAN;
+          // This means that there is nothing else to parse. Apparently
+          // the user wants all isotopologues and no frequency limits.
+          misotopologue = spr.Isotopologue().nelem();
+          return;
+        }
     }
-    
+
   // Check for joker:
   if ( "*" == isoname )
     {
-      // The user wants all isotopes. Set this accordingly:
-      misotope = spr.Isotope().nelem();
+      // The user wants all isotopologues. Set this accordingly:
+      misotopologue = spr.Isotopologue().nelem();
     }
-  else if ( "nl" == isoname )     // Check for "nl":
+//  else if ( "nl" == isoname )     // Check for "nl":
+//    {
+//      // The user wants no lines at all. Set this accordingly:
+//      misotopologue = -1;
+//    }
+  else if ( "CIA" == isoname )     // Check for "cia":
     {
-      // The user wants no lines at all. Set this accordingly:
-      misotope = -1;
+      // The user wants this to use the CIA catalog:
+      mtype = TYPE_CIA;
+      misotopologue = -1;
+
+      // We have to read in the second species, and the dataset index
+      n    = def.find('-');    // find the '-'
+
+      if (n == def.npos )
+        {
+          ostringstream os;
+          os << "Invalid species tag " << def_original << ".\n"
+             << "I am missing a minus sign (and a dataset index after that.)";
+          throw runtime_error(os.str());
+        }
+
+      String otherspec = def.substr(0,n);    // Extract before '-'
+      def.erase(0,n+1);                      // Remove from def
+
+    
+      mcia_second = species_index_from_species_name(otherspec);
+      
+      if ( 0 > mcia_second )
+        {
+          ostringstream os;
+          os << "CIA species \"" << otherspec << "\" is not a valid species.";
+          throw runtime_error(os.str());
+        }
+
+      // Convert remaining def to dataset index.
+      
+      // Check that everything remaining is just numbers.
+      for (Index i=0; i<def.nelem(); ++i)
+          if (!isdigit(def[i])) {
+              ostringstream os;
+              os << "Invalid species tag " << def_original << ".\n"
+                 << "The tag should end with a dataset index";
+              throw runtime_error(os.str());
+          }
+      
+      // Do the conversion from string to index:
+      istringstream is(def);
+      is >> mcia_dataset;
+
+      def = "";
     }
   else
     {
-      // Make an array containing the isotope names:
+      // Make an array containing the isotopologue names:
       ArrayOfString ins;
-      for ( Index i=0; i<spr.Isotope().nelem(); ++i )
-        ins.push_back( spr.Isotope()[i].Name() );
+      for ( Index i=0; i<spr.Isotopologue().nelem(); ++i )
+        ins.push_back( spr.Isotopologue()[i].Name() );
 
-      misotope = find_first (ins, isoname);
+      misotopologue = find_first (ins, isoname);
 
-      // Check if we found a matching isotope:
-      if ( misotope < 0 ) 
+      // Check if we found a matching isotopologue:
+      if ( misotopologue < 0 ) 
         {
           ostringstream os;
-          os << "Isotope " << isoname << " is not a valid isotope or "
+          os << "Isotopologue " << isoname << " is not a valid isotopologue or "
              << "absorption model for species " << name << ".\n"
              << "Valid options are:\n";
           for ( Index i=0; i<ins.nelem(); ++i )
             os << name << "-" << ins[i] << "\n";
           throw runtime_error(os.str());
         }
+      
+      // Check if the found isotopologue represents a predefined model
+      // (continuum or full absorption model) and set the type accordingly:
+      if ( !isdigit(isoname[0]) )
+          mtype = TYPE_PREDEF;
     }
 
   if ( 0 == def.nelem() )
@@ -161,9 +276,40 @@ SpeciesTag::SpeciesTag(String def)
       return;
     }
 
+  // Check if a line mixing type is present
+
+  if (def.substr(0, 2) == "LM")
+  {
+      n = def.find('-');    // find the '-' or the end
+      if (n == def.npos) n = def.nelem();
+
+      String lmtype = def.substr(0, n);
+      if (lmtype == "LM_2NDORDER")
+          mline_mixing_type = LINE_MIXING_TYPE_2NDORDER;
+      else if (lmtype == "LM_NONE")
+          mline_mixing_type = LINE_MIXING_TYPE_NONE;
+      else
+      {
+          ostringstream os;
+          os << "Unknown line mixing type \"" << lmtype << "\"";
+          throw runtime_error(os.str());
+      }
+
+      def.erase(0, n+1);
+
+      // Return if there's nothing else to parse
+      if (!def.nelem()) return;
+  }
+
+  if (def[0] != '*' && !isdigit(def[0]))
+    {
+      ostringstream os;
+      os << "Expected frequency limits, but got \"" << def << "\"";
+      throw runtime_error(os.str());
+    }
 
   // Look for the two frequency limits:
-  
+
   // Extract first frequency
   n    = def.find('-');    // find the '-'
   if (n != def.npos )
@@ -179,11 +325,23 @@ SpeciesTag::SpeciesTag(String def)
           // The default for mlf is already -1, meaning `ALL'.
           // So there is nothing to do here.
         }
+      else if (!isdigit(fname[0]))
+        {
+          ostringstream os;
+          os << "Expected frequency limit, but got \"" << fname << "\"";
+          throw runtime_error(os.str());
+        }
       else
         {
           // Convert to Numeric:
-          istringstream is(fname);
-          is >> mlf;
+          char *endptr;
+          mlf = strtod(fname.c_str(), &endptr);
+          if (endptr != fname.c_str() + fname.nelem())
+            {
+              ostringstream os;
+              os << "Error parsing frequency limit \"" << fname << "\"";
+              throw runtime_error(os.str());
+            }
         }
     }
   else
@@ -202,11 +360,23 @@ SpeciesTag::SpeciesTag(String def)
       // The default for muf is already -1, meaning `ALL'.
       // So there is nothing to do here.
     }
+  else if (!isdigit(def[0]))
+    {
+      ostringstream os;
+      os << "Expected frequency limit, but got \"" << def << "\"";
+      throw runtime_error(os.str());
+    }
   else
     {
       // Convert to Numeric:
-      istringstream is(def);
-      is >> muf;
+      char *endptr;
+      muf = strtod(def.c_str(), &endptr);
+      if (endptr != def.c_str() + def.nelem())
+        {
+          ostringstream os;
+          os << "Error parsing frequency limit \"" << def << "\"";
+          throw runtime_error(os.str());
+        }
     }
 }
 
@@ -226,71 +396,104 @@ SpeciesTag::SpeciesTag(String def)
 */
 String SpeciesTag::Name() const 
 {
-  // Species lookup data:
-  extern const Array<SpeciesRecord> species_data;
-  // A reference to the relevant record of the species data:
-  const  SpeciesRecord& spr = species_data[mspecies];
-  // For return value:
-  ostringstream os;
+    // Species lookup data:
+    using global_data::species_data;
+    // A reference to the relevant record of the species data:
+    const  SpeciesRecord& spr = species_data[mspecies];
+    // For return value:
+    ostringstream os;
+    
+    // First the species name:
+    os << spr.Name() << "-";
+    
+    // Is this a CIA tag?
+    if (mtype==TYPE_CIA)
+      {
+        os << "CIA-"
+           << species_name_from_species_index(mcia_second) << "-"
+           << mcia_dataset;
+        
+      }
+    else if (mtype == TYPE_FREE_ELECTRONS || mtype == TYPE_PARTICLES)
+      {
+        os << spr.Name();
+      }
+    else
+      {
+        // Zeeman flag.
+        if (mtype==TYPE_ZEEMAN) os << "Z-";
+        
+        // Now the isotopologue. Can be a single isotopologue or ALL.
+        if ( misotopologue == spr.Isotopologue().nelem() )
+          {
+            // One larger than allowed means all isotopologues!
+            os << "*-";
+          }
+        else if ( misotopologue == -1 )
+          {
+            // -1 means no lines!
+            os << "nl-";
+          }
+        else
+          {
+            os << spr.Isotopologue()[misotopologue].Name() << "-";
+          }
 
-  // First the species name:
-  os << spr.Name() << "-";
+        // Line Mixing Type
 
-  // Now the isotope. Can be "nl", a single isotope or ALL.
-  if ( misotope == spr.Isotope().nelem() )
-    {
-      // One larger than allowed means all isotopes!
-      os << "*-";
-    }
-  else if ( misotope == -1 )
-    {
-      // -1 means no lines!
-      os << "nl-";
-    }
-  else
-    {
-      os << spr.Isotope()[misotope].Name() << "-";
-    }
+        if (mline_mixing_type != LINE_MIXING_TYPE_NONE)
+        {
+            os << "LM_";
+            switch (mline_mixing_type)
+            {
+                case LINE_MIXING_TYPE_2NDORDER:
+                    os << "2NDORDER";
+                    break;
+                default:
+                    throw runtime_error("Invalid line mixing type. This is impossible.");
+            }
+            os << "-";
+        }
 
-  // Now the frequency limits, if there are any. For this we first
-  // need to determine the floating point precision.
-
-  // Determine the precision, depending on whether Numeric is double
-  // or float:  
-  int precision;
+        // Now the frequency limits, if there are any. For this we first
+        // need to determine the floating point precision.
+        
+        // Determine the precision, depending on whether Numeric is double
+        // or float:
+        int precision;
 #ifdef USE_FLOAT
-  precision = FLT_DIG;
+        precision = FLT_DIG;
 #else
 #ifdef USE_DOUBLE
-  precision = DBL_DIG;
+        precision = DBL_DIG;
 #else
 #error Numeric must be double or float
 #endif
 #endif
-
-  if ( 0 > mlf )
-    {
-      // mlf < 0 means no lower limit.
-      os << "*-";
-    }
-  else
-    {
-      os << setprecision(precision);
-      os << mlf << "-";
-    }
-
-  if ( 0 > muf )
-    {
-      // muf < 0 means no upper limit.
-      os << "*";
-    }
-  else
-    {
-      os << setprecision(precision);
-      os << muf;
-    }
-
-  return os.str();
+        
+        if ( 0 > mlf )
+          {
+            // mlf < 0 means no lower limit.
+            os << "*-";
+          }
+        else
+          {
+            os << setprecision(precision);
+            os << mlf << "-";
+          }
+        
+        if ( 0 > muf )
+          {
+            // muf < 0 means no upper limit.
+            os << "*";
+          }
+        else
+          {
+            os << setprecision(precision);
+            os << muf;
+          }
+      }
+    return os.str();
 }
 
 ostream& operator << (ostream& os, const SpeciesTag& ot)
@@ -348,9 +551,6 @@ String get_tag_group_name( const ArrayOfSpeciesTag& tg )
 */
 String get_species_name( const ArrayOfSpeciesTag& tg )
 {
-  // Species lookup data:
-  extern const Array<SpeciesRecord> species_data;
-
   // Get species index of first tag:
   Index spec_ind = tg[0].Species();
 
@@ -367,11 +567,8 @@ String get_species_name( const ArrayOfSpeciesTag& tg )
           throw runtime_error( os.str() );
         }
     }
-  
-  // A reference to the relevant record of the species data:
-  const  SpeciesRecord& spr = species_data[spec_ind];
 
-  return spr.Name();
+  return species_name_from_species_index( spec_ind );
 }
 
 //! Find first occurrence of species in tag groups.
@@ -435,63 +632,6 @@ Index find_next_species_tg( const ArrayOfArrayOfSpeciesTag& tgs,
   return -1;
 }
 
-//! Return species index for given species name.
-/*! 
-  This is useful in connection with other functions that need a species
-  index.
-
-  \see find_first_species_tg.
-
-  \param name Species name.
-
-  \return Species index, -1 means not found.
-
-  \author Stefan Buehler
-  \date   2003-01-13
-*/
-Index species_index_from_species_name( String name )
-{
-  // The species map. This is used to find the species id.
-  extern map<String, Index> SpeciesMap;
-
-  // For the return value:
-  Index mspecies;
-  
-  // Remove leading whitespace, if there is any:
-  while ( 0 != name.nelem() && (
-                                ' '  == name[0] ||
-                                '\t' == name[0] ||
-                                '\n' == name[0] ||
-                                '\r' == name[0]
-                                )
-          )    name.erase(0,1);
-
-  // Remove trailing whitespace, if there is any:
-  while ( 0 != name.nelem() && (
-                                ' '  == name[name.nelem()-1] ||
-                                '\t' == name[name.nelem()-1] ||
-                                '\n' == name[name.nelem()-1] ||
-                                '\r' == name[name.nelem()-1]
-                                )
-          )    name.erase(name.nelem()-1);
-
-  //  cout << "name / def = " << name << " / " << def << endl;
-
-  // Look for species name in species map:
-  map<String, Index>::const_iterator mi = SpeciesMap.find(name);
-  if ( mi != SpeciesMap.end() )
-    {
-      // Ok, we've found the species. Set mspecies.
-      mspecies = mi->second;
-    }
-  else
-    {
-      // The species does not exist!
-      mspecies = -1;
-    }
-
-  return mspecies;
-}
 
 //! Converts a String to ArrayOfSpeciesTag
 /*!
@@ -541,83 +681,107 @@ void array_species_tag_from_string( ArrayOfSpeciesTag& tags,
   {
     SpeciesTag this_tag(tag_def[s]);
 
-    // Safety check: For s>0 check that the tags belong to the same species.
+    // Safety checks:
     if (s>0)
-    if ( tags[0].Species() != this_tag.Species() )
-      throw runtime_error("Tags in a tag group must belong to the same species.");
+      {
+        // Tags inside a group must belong to the same species.
+        if ( tags[0].Species() != this_tag.Species() )
+            throw runtime_error("Tags in a tag group must belong to the same species.");
 
+        // Zeeman tags and plain line by line tags must not be mixed. (Because
+        // there can be only one line list per tag group.)
+        if (
+            ((tags[0].Type()==SpeciesTag::TYPE_ZEEMAN) &&
+             (this_tag.Type()==SpeciesTag::TYPE_PLAIN))
+            ||
+            ((tags[0].Type()==SpeciesTag::TYPE_PLAIN) &&
+             (this_tag.Type()==SpeciesTag::TYPE_ZEEMAN))
+            )
+            throw runtime_error("Zeeman tags and plain line-by-line tags must "
+                                "not be mixed in the same tag group.");
+      }
+    
     tags.push_back(this_tag);
   }
 }
-   
-/** 
-   Returns the index among some tag groups for an array of tag Strings. 
-   
-   \verbatim
-   For example, if tags1 correspond to the definition
-     ["O3","H2O-161,H2O-162"]
-   and the tag Strings are
-     ["H2O-161,H2O-162","O3"]
-   the tags1_index becomes
-     [2,1]
-   \endverbatim
 
-   @exception runtime_error  Some String is not a valid tag item.
-   @exception runtime_error  Not all Strings are not found among the tags.
 
-   \retval tags1_index     Index in tags1 for tags2_Strings
-   \param  tags1           The tags to search in.
-   \param  tags2_Strings   The tag Strings for which indeces shall be found.
+//! Check the correctness of abs_species.
+/*!
+   Checks on the correctness of the tags will be performed,
+   e.g. free_electrons and particles species are only allowed once in
+   abs_species.
 
-   \author Patrick Eriksson 
-   \date 2000-12-06
+   \param tags  Array of Array of SpeciesTag.
+
+   \author Oliver Lemke
+   \date   2013-04-23
 */
-void get_tagindex_for_Strings( 
-                              ArrayOfIndex&   tags1_index, 
-                              const ArrayOfArrayOfSpeciesTag&      tags1, 
-                              const ArrayOfString&  tags2_Strings,
-                              const Verbosity& verbosity)
+void check_abs_species( const ArrayOfArrayOfSpeciesTag& abs_species )
 {
-  const Index   n1 = tags1.nelem();
-  const Index   n2 = tags2_Strings.nelem();
-  ArrayOfArrayOfSpeciesTag   tags2;                // Internal tag names for tag_Strings
-  Index   i1, i2, nj, j, found, ok;
-
-  tags1_index.resize(n2);
-  //  cout << "tags2_Strings: " << tags2_Strings << "\n";
-  SpeciesSet( tags2, tags2_Strings, verbosity);
-
-  for ( i2=0; i2<n2; i2++ )
-  {
-    found = 0;
-    for ( i1=0; (i1<n1) && !found; i1++ )
+    Index num_free_electrons = 0;
+    for ( Index i=0; i<abs_species.nelem(); ++i )
     {
-      nj = tags2[i2].nelem(); 
-      if ( nj  == tags1[i1].nelem() )
-      {
-        ok = 1;
-        for ( j=0; j<nj; j++ )
+        bool has_free_electrons = false;
+        bool has_particles = false;
+        bool has_line_mixing = false;
+        for ( Index s=0; s<abs_species[i].nelem(); ++s )
         {
-          if ( tags2[i2][j].Name() != tags1[i1][j].Name() )
-            ok = 0;
+            if (abs_species[i][s].Type() == SpeciesTag::TYPE_FREE_ELECTRONS)
+            {
+                num_free_electrons++;
+                has_free_electrons = true;
+            }
+
+            if (abs_species[i][s].Type() == SpeciesTag::TYPE_PARTICLES)
+            {
+                has_particles = true;
+            }
+
+            if (abs_species[i][s].LineMixingType() != SpeciesTag::LINE_MIXING_TYPE_NONE)
+            {
+                has_line_mixing = true;
+            }
         }
-        if ( ok )
-        {
-          found = 1;
-          tags1_index[i2] = i1;
-        }
-      }
-    } 
-    if ( !found )
-    {
-      ostringstream os;
-      os << "The tag String \"" << tags2_Strings[i2] << 
-            "\" does not match any of the given tags.\n";
-      throw runtime_error(os.str());
+
+        if (abs_species[i].nelem() > 1 && has_free_electrons)
+            throw runtime_error("'free_electrons' must not be combined "
+                                "with other tags in the same group.");
+        if (num_free_electrons > 1)
+            throw runtime_error("'free_electrons' must not be defined "
+                                "more than once.");
+
+        if (abs_species[i].nelem() > 1 && has_particles)
+            throw runtime_error("'particles' must not be combined "
+                                "with other tags in the same group.");
+        if (abs_species[i].nelem() > 1 && has_line_mixing)
+            throw runtime_error("Line mixing species must not be combined "
+                                "with other tags in the same group.");
     }
-  }
 }
 
+
+/**
+ Is this a Zeeman tag group?
+
+ It is not enough to just look at the first tag to find out if this is a Zeeman
+ tag group, since there could be some predefined absorption or CIA tag first.
+ 
+ \return true if this is a Zeeman tag group, otherwise false.
+ \param  tg  The tag group to check
+ 
+ \author Stefan Buehler
+ \date   2013-01-15 */
+bool is_zeeman(const ArrayOfSpeciesTag& tg)
+{
+    for ( Index s=0; s<tg.nelem(); ++s )
+        if (tg[s].Type()==SpeciesTag::TYPE_ZEEMAN)
+            return true;
+        
+    return false;
+}
+
+
 /** Returns the index of the tag group tg2 within the array of tag
     groups tgs1. Slightly modified copy of get_tagindex_for_Strings.
    
