@@ -1,4 +1,4 @@
-/* Copyright (C) 2008 Stefan Buehler <sbuehler(at)ltu.se>
+/* Copyright (C) 2012 Stefan Buehler <sbuehler(at)ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -37,11 +37,14 @@
 */
 
 #include <iostream>
+#include <cmath>
 #include "interpolation_poly.h"
 #include "interpolation.h"
 #include "logic.h"
+#include "check_input.h"
 #include "arts_omp.h"
 
+
 //! Return the maximum of two integer numbers.
 /*! 
   This function is based on a macro from Numerical Receipes. The
@@ -106,7 +109,7 @@ const Numeric sum_check_epsilon = 1e-6;
 /*!
   This function performs the same task as gridpos, but for arbitrary
   orders of interpolation. (Linear is also possible, as a special
-  case.) 
+  case.) We even handle nearest neighbour interpolation, as order 0.
   
   The formula for calculating the weights w is taken from Numerical
   Recipes, 2nd edition, section 3.1, eq. 3.1.1.
@@ -115,7 +118,7 @@ const Numeric sum_check_epsilon = 1e-6;
   \param old_grid Original grid.
   \param new_grid New grid.
   \param order Interpolation order.
-               1 = linear, 2 = quadratic, etc..
+               0 = nearest neighbour, 1 = linear, 2 = quadratic, etc..
                The number of points used in the
                interpolation will be order+1.
   \param extpolfac Extrapolation fraction. Should normally not be
@@ -143,8 +146,28 @@ void gridpos_poly(ArrayOfGridPosPoly& gp,
   
   // First call the traditional gridpos to find the grid positions:
   ArrayOfGridPos gp_trad(n_new);
-  gridpos( gp_trad, old_grid, new_grid, extpolfac );
-
+  if (n_old>1)
+    {
+      gridpos( gp_trad, old_grid, new_grid, extpolfac );
+    }
+  else if (n_old==1)
+    {
+      // This case is not handled by traditional gridpos, but is ok for zero
+      // order interpolation, so we handle it explicitly here. Since there is
+      // only 1 point in the old grid, it is nearest neighbour to all
+      // interpolation points.
+      for (Index i=0; i<n_new; ++i) {
+          gp_trad[i].idx = 0;
+          gp_trad[i].fd[0] = 0;
+          gp_trad[i].fd[1] = 1;
+      }
+    }
+  else
+    {
+      // We should never be here.
+      assert(false);
+    }
+    
   for (Index s=0; s<n_new; ++s)
     {
                    
@@ -152,9 +175,28 @@ void gridpos_poly(ArrayOfGridPosPoly& gp,
       // points used for interpolation. For linear interpolation this
       // is identical to j. The idea for this expression is from
       // Numerical Receipes (Chapter 3, section "after the hunt"), but
-      // there is is for 1-based arrays.
-      Index k = IMIN(IMAX(gp_trad[s].idx-(m-1)/2, 0),
+      // there it is for 1-based arrays.
+      Index k;
+      if (m!=1)
+        {
+          k = IMIN(IMAX(gp_trad[s].idx-(m-1)/2, 0),
                      n_old-m);
+        }
+      else
+        {
+          // The above formula for k is not valid for m==1
+          // (nearest neighbour interpolation).
+          if (gp_trad[s].fd[0]<=0.5)
+              k=gp_trad[s].idx;
+          else
+              k=gp_trad[s].idx+1;
+
+          // It is a matter of definition what we do with the exact fd==0.5 case.
+          // Here I arbitrarily decided to stick with the "left" point (smaller
+          // index). I believe this is consistent with the behaviour for m=3,
+          // where 2 points on the left and 1 point on the right is used. (So,
+          // we always prefer the left side.)
+        }
 
       //      cout << "m: "<< m << ", k: " << k << endl;
 
@@ -233,11 +275,118 @@ void gridpos_poly( GridPosPoly&    gp,
 }
 
 
+//! Set up grid positions for higher order interpolation on longitudes.
+/*!
+ This function performs the same task as gridpos, but for arbitrary
+ orders of interpolation. It is especially for interpolation of longitudes.
+ If necessary, it will shift the given longitudes up or down by 360 degrees.
+ For a global longitude grid, it will do cyclic interpolation.
+
+ The formula for calculating the weights w is taken from Numerical
+ Recipes, 2nd edition, section 3.1, eq. 3.1.1.
+
+ \param[in]  error_msg  Message for potential runtime error
+ \param[out] gp         An array of grid positions.
+ \param[in]  old_grid   Original grid.
+ \param[in]  new_grid   New grid.
+ \param[in]  order      Interpolation order.
+                        1 = linear, 2 = quadratic, etc..
+                        The number of points used in the
+                        interpolation will be order+1.
+ \param[in]  extpolfac  Extrapolation fraction. Should normally not be
+                        specified, then the default of 0.5 is used.
+
+ \author Oliver Lemke
+ \date 12-07-10
+ */
+void gridpos_poly_longitudinal(const String&   error_msg,
+                               ArrayOfGridPosPoly& gp,
+                               ConstVectorView old_grid,
+                               ConstVectorView new_grid,
+                               const Index     order,
+                               const Numeric&  extpolfac)
+{
+    bool global_lons = is_lon_cyclic(old_grid);
+
+    if (global_lons)
+        gridpos_poly_cyclic_longitudinal(gp, old_grid, new_grid, order, extpolfac);
+    else
+    {
+        if (old_grid[0] >= new_grid[new_grid.nelem()-1])
+        {
+            Vector shifted_old_grid = old_grid;
+            shifted_old_grid -= 360;
+            chk_interpolation_grids(error_msg, shifted_old_grid, new_grid, order, extpolfac);
+
+            gridpos_poly(gp, shifted_old_grid, new_grid, order, extpolfac);
+        }
+        else if (old_grid[old_grid.nelem()-1] <= new_grid[0])
+        {
+            Vector shifted_old_grid = old_grid;
+            shifted_old_grid += 360;
+            chk_interpolation_grids(error_msg, shifted_old_grid, new_grid, order, extpolfac);
+            gridpos_poly(gp, shifted_old_grid, new_grid, order, extpolfac);
+        }
+        else
+        {
+            chk_interpolation_grids(error_msg, old_grid, new_grid, order, extpolfac);
+            gridpos_poly(gp, old_grid, new_grid, order, extpolfac);
+        }
+    }
+}
 
 
+//! Set up grid positions for higher order interpolation.
+/*!
+ This function performs the same task as gridpos, but for arbitrary
+ orders of interpolation. It is especially for cyclic interpolation of longitudes.
+ The distance between the first and the last point in the original grid
+ has to span 360 degrees.
+
+ \param[out] gp         An array of grid positions.
+ \param[in]  old_grid   Original grid.
+ \param[in]  new_grid   New grid.
+ \param[in]  order      Interpolation order.
+                        1 = linear, 2 = quadratic, etc..
+                        The number of points used in the
+                        interpolation will be order+1.
+ \param[in]  extpolfac  Extrapolation fraction. Should normally not be
+                        specified, then the default of 0.5 is used.
+
+ \author Oliver Lemke
+ \date 12-07-10
+ */
+void gridpos_poly_cyclic_longitudinal(ArrayOfGridPosPoly& gp,
+                                      ConstVectorView old_grid,
+                                      ConstVectorView new_grid,
+                                      const Index     order,
+                                      const Numeric&  extpolfac)
+{
+    assert(is_lon_cyclic(old_grid));
+
+    Index new_n = old_grid.nelem() - 1;
+    ConstVectorView old_grid_n1 = old_grid[Range(0, new_n)];
+    Range r_left = Range(0, new_n);
+    Range r_right = Range(new_n*2, new_n);
+
+    Vector large_grid(new_n*3);
+
+    large_grid[r_left] = old_grid_n1;
+    large_grid[r_left] -= 360.;
+
+    large_grid[Range(new_n, new_n)] = old_grid_n1;
+
+    large_grid[r_right] = old_grid_n1;
+    large_grid[r_right] += 360.;
+
+    gridpos_poly(gp, large_grid, new_grid, order, extpolfac);
+
+    for (ArrayOfGridPosPoly::iterator itgp = gp.begin(); itgp != gp.end(); itgp++)
+        for (ArrayOfIndex::iterator iti = itgp->idx.begin(); iti != itgp->idx.end(); iti++)
+            *iti %= new_n;
+}
 
 
-// FIXME: Below here is copied code from interpolation.cc that must be adapted.
 
 //! Macro for interpolation weight loops.
 /*!
@@ -254,9 +403,11 @@ void gridpos_poly( GridPosPoly&    gp,
   for ( ConstIterator1D x=tx.w.begin(); x!=tx.w.end(); ++x )
 
 */
-//#define LOOPW(x) for ( const Numeric* x=&t##x.fd[1]; x>=&t##x.fd[0]; --x )
-//#define LOOPW(x) for ( Index x=0; x<t##x.w.nelem(); ++x )
-#define LOOPW(x) for ( ConstIterator1D x=t##x.w.begin(); x!=t##x.w.end(); ++x )
+#define LOOPW(x) for ( ConstIterator1D x = t##x##begin; x!=t##x##end; ++x)
+
+//! Macro for caching begin and end iterators for interpolation weight loops.
+#define CACHEW(x) const ConstIterator1D t##x##begin = t##x.w.begin(); \
+                  const ConstIterator1D t##x##end = t##x.w.end();
 
 //! Macro for interpolation index loops.
 /*!
@@ -264,7 +415,11 @@ void gridpos_poly( GridPosPoly&    gp,
   is an ArrayOfIndex, not a Vector, we have to use a different type of
   iterator.
 */
-#define LOOPIDX(x) for (ArrayOfIndex::const_iterator x=t##x.idx.begin(); x!=t##x.idx.end(); ++x)
+#define LOOPIDX(x) for (ArrayOfIndex::const_iterator x=t##x##begin; x!=t##x##end; ++x)
+
+//! Macro for caching begin and end iterators for interpolation index loops.
+#define CACHEIDX(x) const ArrayOfIndex::const_iterator t##x##begin = t##x.idx.begin(); \
+                    const ArrayOfIndex::const_iterator t##x##end = t##x.idx.end();
 
 
 //! Output operator for GridPosPoly.
@@ -324,9 +479,10 @@ void interpweights( VectorView itw,
 
   Index iti = 0;
 
+  CACHEW(c)
   LOOPW(c)
     {
-      itw[iti] = *c;
+      itw.get(iti) = *c;
       ++iti;
     }
 }
@@ -355,10 +511,12 @@ void interpweights( VectorView itw,
                  tc.w.nelem())); 
   Index iti = 0;
 
+  CACHEW(r)
+  CACHEW(c)
   LOOPW(r)
   LOOPW(c)
     {
-      itw[iti] = (*r) * (*c);
+      itw.get(iti) = (*r) * (*c);
       ++iti;
     }
 }
@@ -391,11 +549,14 @@ void interpweights( VectorView itw,
                                
   Index iti = 0;
 
+  CACHEW(p)
+  CACHEW(r)
+  CACHEW(c)
   LOOPW(p)
   LOOPW(r)
   LOOPW(c)
     {
-      itw[iti] = (*p) * (*r) * (*c);
+      itw.get(iti) = (*p) * (*r) * (*c);
       ++iti;
     }
 }
@@ -431,12 +592,16 @@ void interpweights( VectorView itw,
                                 
   Index iti = 0;
 
+  CACHEW(b)
+  CACHEW(p)
+  CACHEW(r)
+  CACHEW(c)
   LOOPW(b)
   LOOPW(p)
   LOOPW(r)
   LOOPW(c)
     {
-      itw[iti] = (*b) * (*p) * (*r) * (*c);
+      itw.get(iti) = (*b) * (*p) * (*r) * (*c);
       ++iti;
     }
 }
@@ -475,13 +640,18 @@ void interpweights( VectorView itw,
                                 
   Index iti = 0;
 
+  CACHEW(s)
+  CACHEW(b)
+  CACHEW(p)
+  CACHEW(r)
+  CACHEW(c)
   LOOPW(s)
   LOOPW(b)
   LOOPW(p)
   LOOPW(r)
   LOOPW(c)
     {
-      itw[iti] = (*s) * (*b) * (*p) * (*r) * (*c);
+      itw.get(iti) = (*s) * (*b) * (*p) * (*r) * (*c);
       ++iti;
     }
 }
@@ -523,6 +693,12 @@ void interpweights( VectorView itw,
                                 
   Index iti = 0;
 
+  CACHEW(v)
+  CACHEW(s)
+  CACHEW(b)
+  CACHEW(p)
+  CACHEW(r)
+  CACHEW(c)
   LOOPW(v)
   LOOPW(s)
   LOOPW(b)
@@ -530,7 +706,7 @@ void interpweights( VectorView itw,
   LOOPW(r)
   LOOPW(c)
     {
-      itw[iti] = (*v) * (*s) * (*b) * (*p) * (*r) * (*c);
+      itw.get(iti) = (*v) * (*s) * (*b) * (*p) * (*r) * (*c);
       ++iti;
     }
 }
@@ -567,9 +743,10 @@ Numeric interp( ConstVectorView itw,
   Numeric tia = 0;
 
   Index iti = 0;
+  CACHEIDX(c)
   LOOPIDX(c)
     {
-      tia += a[*c] * itw[iti];
+      tia += a.get(*c) * itw.get(iti);
       ++iti;
     }
 
@@ -612,11 +789,13 @@ Numeric interp( ConstVectorView  itw,
   Numeric tia = 0;
 
   Index iti = 0;
+  CACHEIDX(r)
+  CACHEIDX(c)
   LOOPIDX(r)
     LOOPIDX(c)
       {
-        tia += a(*r,
-                 *c) * itw[iti];
+        tia += a.get(*r,
+                     *c) * itw.get(iti);
         ++iti;
       }
 
@@ -662,13 +841,16 @@ Numeric interp( ConstVectorView  itw,
   Numeric tia = 0;
 
   Index iti = 0;
+  CACHEIDX(p)
+  CACHEIDX(r)
+  CACHEIDX(c)
   LOOPIDX(p)
   LOOPIDX(r)
   LOOPIDX(c)
         {
-          tia += a(*p,
-                   *r,
-                   *c) * itw[iti];
+          tia += a.get(*p,
+                       *r,
+                       *c) * itw.get(iti);
           ++iti;
         }
 
@@ -717,15 +899,19 @@ Numeric interp( ConstVectorView  itw,
   Numeric tia = 0;
 
   Index iti = 0;
+  CACHEIDX(b)
+  CACHEIDX(p)
+  CACHEIDX(r)
+  CACHEIDX(c)
   LOOPIDX(b)
   LOOPIDX(p)
   LOOPIDX(r)
   LOOPIDX(c)
           {
-            tia += a(*b,
-                     *p,
-                     *r,
-                     *c) * itw[iti];
+            tia += a.get(*b,
+                         *p,
+                         *r,
+                         *c) * itw.get(iti);
             ++iti;
           }
 
@@ -777,17 +963,22 @@ Numeric interp( ConstVectorView  itw,
   Numeric tia = 0;
 
   Index iti = 0;
+  CACHEIDX(s)
+  CACHEIDX(b)
+  CACHEIDX(p)
+  CACHEIDX(r)
+  CACHEIDX(c)
   LOOPIDX(s)
   LOOPIDX(b)
   LOOPIDX(p)
   LOOPIDX(r)
   LOOPIDX(c)
             {
-              tia += a(*s,
-                       *b,
-                       *p,
-                       *r,
-                       *c) * itw[iti];
+              tia += a.get(*s,
+                           *b,
+                           *p,
+                           *r,
+                           *c) * itw.get(iti);
               ++iti;
             }
 
@@ -842,6 +1033,12 @@ Numeric interp( ConstVectorView  itw,
   Numeric tia = 0;
 
   Index iti = 0;
+  CACHEIDX(v)
+  CACHEIDX(s)
+  CACHEIDX(b)
+  CACHEIDX(p)
+  CACHEIDX(r)
+  CACHEIDX(c)
   LOOPIDX(v)
   LOOPIDX(s)
   LOOPIDX(b)
@@ -849,12 +1046,12 @@ Numeric interp( ConstVectorView  itw,
   LOOPIDX(r)
   LOOPIDX(c)
               {
-                tia += a(*v,
-                         *s,
-                         *b,
-                         *p,
-                         *r,
-                         *c) * itw[iti];
+                tia += a.get(*v,
+                             *s,
+                             *b,
+                             *p,
+                             *r,
+                             *c) * itw.get(iti);
                 ++iti;
               }
 
@@ -927,9 +1124,10 @@ void interpweights( MatrixView itw,
       // to the for loop above. Note: NO SEMICOLON AFTER THE LOOPW
       // COMMAND! 
 
+      CACHEW(c)
       LOOPW(c)
         {
-          itw(i,iti) = *c;
+          itw.get(i,iti) = *c;
           ++iti;
         }
     }
@@ -984,10 +1182,12 @@ void interpweights( MatrixView itw,
 
       Index iti = 0;
 
+      CACHEW(r)
+      CACHEW(c)
       LOOPW(r)
       LOOPW(c)
           {
-            itw(i,iti) = (*r) * (*c);
+            itw.get(i,iti) = (*r) * (*c);
             ++iti;
           }
     }
@@ -1039,11 +1239,14 @@ void interpweights( MatrixView itw,
       const GridPosPoly& tc = cgp[i];
 
       Index iti = 0;
+      CACHEW(p)
+      CACHEW(r)
+      CACHEW(c)
       LOOPW(p)
       LOOPW(r)
       LOOPW(c)
         {
-          itw(i,iti) = (*p) * (*r) * (*c);
+          itw.get(i,iti) = (*p) * (*r) * (*c);
           ++iti;
         }
     }
@@ -1100,12 +1303,16 @@ void interpweights( MatrixView itw,
       const GridPosPoly& tc = cgp[i];
 
       Index iti = 0;
+      CACHEW(b)
+      CACHEW(p)
+      CACHEW(r)
+      CACHEW(c)
       LOOPW(b)
       LOOPW(p)
       LOOPW(r)
       LOOPW(c)
         {
-          itw(i,iti) = (*b) * (*p) * (*r) * (*c);
+          itw.get(i,iti) = (*b) * (*p) * (*r) * (*c);
           ++iti;
         }
     }
@@ -1167,13 +1374,18 @@ void interpweights( MatrixView itw,
       const GridPosPoly& tc = cgp[i];
 
       Index iti = 0;
+      CACHEW(s)
+      CACHEW(b)
+      CACHEW(p)
+      CACHEW(r)
+      CACHEW(c)
       LOOPW(s)
       LOOPW(b)
       LOOPW(p)
       LOOPW(r)
       LOOPW(c)
         {
-          itw(i,iti) = (*s) * (*b) * (*p) * (*r) * (*c);
+          itw.get(i,iti) = (*s) * (*b) * (*p) * (*r) * (*c);
           ++iti;
         }
     }
@@ -1240,6 +1452,12 @@ void interpweights( MatrixView itw,
       const GridPosPoly& tc = cgp[i];
 
       Index iti = 0;
+      CACHEW(v)
+      CACHEW(s)
+      CACHEW(b)
+      CACHEW(p)
+      CACHEW(r)
+      CACHEW(c)
       LOOPW(v)
       LOOPW(s)
       LOOPW(b)
@@ -1247,7 +1465,7 @@ void interpweights( MatrixView itw,
       LOOPW(r)
       LOOPW(c)
         {
-          itw(i,iti) = (*v) * (*s) * (*b) * (*p) * (*r) * (*c);
+          itw.get(i,iti) = (*v) * (*s) * (*b) * (*p) * (*r) * (*c);
           ++iti;
         }
     }
@@ -1299,9 +1517,10 @@ void interp( VectorView            ia,
       tia = 0;
 
       Index iti = 0;
+      CACHEIDX(c)
       LOOPIDX(c)
         {
-          tia += a[*c] * itw(i,iti);
+          tia += a.get(*c) * itw.get(i,iti);
           ++iti;
         }
     }
@@ -1362,11 +1581,13 @@ void interp( VectorView            ia,
       tia = 0;
 
       Index iti = 0;
+      CACHEIDX(r)
+      CACHEIDX(c)
       LOOPIDX(r)
         LOOPIDX(c)
           {
-            tia += a(*r,
-                     *c) * itw(i,iti);
+            tia += a.get(*r,
+                         *c) * itw.get(i,iti);
             ++iti;
           }
     }
@@ -1432,13 +1653,16 @@ void interp( VectorView            ia,
       tia = 0;
 
       Index iti = 0;
+      CACHEIDX(p)
+      CACHEIDX(r)
+      CACHEIDX(c)
       LOOPIDX(p)
       LOOPIDX(r)
       LOOPIDX(c)
             {
-              tia += a(*p,
-                       *r,
-                       *c) * itw(i,iti);
+              tia += a.get(*p,
+                           *r,
+                           *c) * itw.get(i,iti);
               ++iti;
             }
     }
@@ -1509,15 +1733,19 @@ void interp( VectorView            ia,
       tia = 0;
 
       Index iti = 0;
+      CACHEIDX(b)
+      CACHEIDX(p)
+      CACHEIDX(r)
+      CACHEIDX(c)
       LOOPIDX(b)
       LOOPIDX(p)
       LOOPIDX(r)
       LOOPIDX(c)
               {
-                tia += a(*b,
-                         *p,
-                         *r,
-                         *c) * itw(i,iti);
+                tia += a.get(*b,
+                             *p,
+                             *r,
+                             *c) * itw.get(i,iti);
                 ++iti;
               }
     }
@@ -1593,17 +1821,22 @@ void interp( VectorView            ia,
       tia = 0;
 
       Index iti = 0;
+      CACHEIDX(s)
+      CACHEIDX(b)
+      CACHEIDX(p)
+      CACHEIDX(r)
+      CACHEIDX(c)
       LOOPIDX(s)
       LOOPIDX(b)
       LOOPIDX(p)
       LOOPIDX(r)
       LOOPIDX(c)
                 {
-                  tia += a(*s,
-                           *b,
-                           *p,
-                           *r,
-                           *c) * itw(i,iti);
+                  tia += a.get(*s,
+                               *b,
+                               *p,
+                               *r,
+                               *c) * itw.get(i,iti);
                   ++iti;
                 }
     }
@@ -1684,6 +1917,12 @@ void interp( VectorView            ia,
       tia = 0;
 
       Index iti = 0;
+      CACHEIDX(v)
+      CACHEIDX(s)
+      CACHEIDX(b)
+      CACHEIDX(p)
+      CACHEIDX(r)
+      CACHEIDX(c)
       LOOPIDX(v)
       LOOPIDX(s)
       LOOPIDX(b)
@@ -1691,12 +1930,12 @@ void interp( VectorView            ia,
       LOOPIDX(r)
       LOOPIDX(c)
                   {
-                    tia += a(*v,
-                             *s,
-                             *b,
-                             *p,
-                             *r,
-                             *c) * itw(i,iti);
+                    tia += a.get(*v,
+                                 *s,
+                                 *b,
+                                 *p,
+                                 *r,
+                                 *c) * itw.get(i,iti);
                     ++iti;
                   }
     }
@@ -1744,11 +1983,13 @@ void interpweights( Tensor3View itw,
     {
       // Current grid position:
       const GridPosPoly& tr = rgp[ir];
+      CACHEW(r)
 
       for ( Index ic=0; ic<nc; ++ic )
         {
           // Current grid position:
           const GridPosPoly& tc = cgp[ic];
+          CACHEW(c)
 
           // Interpolation weights are stored in this order (l=lower
           // u=upper, r=row, c=column):
@@ -1762,7 +2003,7 @@ void interpweights( Tensor3View itw,
           LOOPW(r)
             LOOPW(c)
             {
-              itw(ir,ic,iti) = (*r) * (*c);
+              itw.get(ir,ic,iti) = (*r) * (*c);
               ++iti;
             }
         }
@@ -1810,12 +2051,15 @@ void interpweights( Tensor4View itw,
   for ( Index ip=0; ip<np; ++ip )
     {
       const GridPosPoly& tp = pgp[ip];
+      CACHEW(p)
       for ( Index ir=0; ir<nr; ++ir )
         {
           const GridPosPoly& tr = rgp[ir];
+          CACHEW(r)
           for ( Index ic=0; ic<nc; ++ic )
             {
               const GridPosPoly& tc = cgp[ic];
+              CACHEW(c)
 
               Index iti = 0;
 
@@ -1823,7 +2067,7 @@ void interpweights( Tensor4View itw,
                 LOOPW(r)
                 LOOPW(c)
                 {
-                  itw(ip,ir,ic,iti) =
+                  itw.get(ip,ir,ic,iti) =
                     (*p) * (*r) * (*c);
                   ++iti;
                 }
@@ -1877,15 +2121,19 @@ void interpweights( Tensor5View itw,
   for ( Index ib=0; ib<nb; ++ib )
     {
       const GridPosPoly& tb = bgp[ib];
+      CACHEW(b)
       for ( Index ip=0; ip<np; ++ip )
         {
           const GridPosPoly& tp = pgp[ip];
+          CACHEW(p)
           for ( Index ir=0; ir<nr; ++ir )
             {
               const GridPosPoly& tr = rgp[ir];
+              CACHEW(r)
               for ( Index ic=0; ic<nc; ++ic )
                 {
                   const GridPosPoly& tc = cgp[ic];
+                  CACHEW(c)
 
                   Index iti = 0;
 
@@ -1894,7 +2142,7 @@ void interpweights( Tensor5View itw,
                     LOOPW(r)
                     LOOPW(c)
                     {
-                      itw(ib,ip,ir,ic,iti) =
+                      itw.get(ib,ip,ir,ic,iti) =
                         (*b) * (*p) * (*r) * (*c);
                       ++iti;
                     }
@@ -1953,18 +2201,23 @@ void interpweights( Tensor6View itw,
   for ( Index is=0; is<ns; ++is )
     {
       const GridPosPoly& ts = sgp[is];
+      CACHEW(s)
       for ( Index ib=0; ib<nb; ++ib )
         {
           const GridPosPoly& tb = bgp[ib];
+          CACHEW(b)
           for ( Index ip=0; ip<np; ++ip )
             {
               const GridPosPoly& tp = pgp[ip];
+              CACHEW(p)
               for ( Index ir=0; ir<nr; ++ir )
                 {
                   const GridPosPoly& tr = rgp[ir];
+                  CACHEW(r)
                   for ( Index ic=0; ic<nc; ++ic )
                     {
                       const GridPosPoly& tc = cgp[ic];
+                      CACHEW(c)
 
                       Index iti = 0;
 
@@ -1974,7 +2227,7 @@ void interpweights( Tensor6View itw,
                         LOOPW(r)
                         LOOPW(c)
                         {
-                          itw(is,ib,ip,ir,ic,iti) =
+                          itw.get(is,ib,ip,ir,ic,iti) =
                             (*s) * (*b) * (*p) * (*r) * (*c);
                           ++iti;
                         }
@@ -2038,21 +2291,27 @@ void interpweights( Tensor7View itw,
   for ( Index iv=0; iv<nv; ++iv )
     {
       const GridPosPoly& tv = vgp[iv];
+      CACHEW(v)
       for ( Index is=0; is<ns; ++is )
         {
           const GridPosPoly& ts = sgp[is];
+          CACHEW(s)
           for ( Index ib=0; ib<nb; ++ib )
             {
               const GridPosPoly& tb = bgp[ib];
+              CACHEW(b)
               for ( Index ip=0; ip<np; ++ip )
                 {
                   const GridPosPoly& tp = pgp[ip];
+                  CACHEW(p)
                   for ( Index ir=0; ir<nr; ++ir )
                     {
                       const GridPosPoly& tr = rgp[ir];
+                      CACHEW(r)
                       for ( Index ic=0; ic<nc; ++ic )
                         {
                           const GridPosPoly& tc = cgp[ic];
+                          CACHEW(c)
 
                           Index iti = 0;
 
@@ -2063,7 +2322,7 @@ void interpweights( Tensor7View itw,
                             LOOPW(r)
                             LOOPW(c)
                             {
-                              itw(iv,is,ib,ip,ir,ic,iti) =
+                              itw.get(iv,is,ib,ip,ir,ic,iti) =
                                 (*v) * (*s) * (*b) * (*p) * (*r) * (*c);
                               ++iti;
                             }
@@ -2122,11 +2381,13 @@ void interp( MatrixView            ia,
     {
       // Current grid position:
       const GridPosPoly& tr = rgp[ir];
+      CACHEIDX(r)
 
       for ( Index ic=0; ic<nc; ++ic )
         {
           // Current grid position:
           const GridPosPoly& tc = cgp[ic];
+          CACHEIDX(c)
 
           // Get handle to current element of output tensor and initialize
           // it to zero:
@@ -2137,8 +2398,8 @@ void interp( MatrixView            ia,
           LOOPIDX(r)
           LOOPIDX(c)
             {
-              tia += a(*r,
-                       *c) * itw(ir,ic,iti);
+              tia += a.get(*r,
+                           *c) * itw.get(ir,ic,iti);
               ++iti;
             }
         }
@@ -2197,13 +2458,16 @@ void interp( Tensor3View           ia,
   for ( Index ip=0; ip<np; ++ip )
     {
       const GridPosPoly& tp = pgp[ip];
+      CACHEIDX(p);
       for ( Index ir=0; ir<nr; ++ir )
         {
           const GridPosPoly& tr = rgp[ir];
+          CACHEIDX(r);
           for ( Index ic=0; ic<nc; ++ic )
             {
               // Current grid position:
               const GridPosPoly& tc = cgp[ic];
+              CACHEIDX(c);
 
               // Get handle to current element of output tensor and
               // initialize it to zero:
@@ -2215,10 +2479,10 @@ void interp( Tensor3View           ia,
               LOOPIDX(r)
               LOOPIDX(c)
                     {
-                      tia += a(*p,
-                               *r,
-                               *c) * itw(ip,ir,ic,
-                                               iti);
+                      tia += a.get(*p,
+                                   *r,
+                                   *c) * itw.get(ip,ir,ic,
+                                                 iti);
                       ++iti;
                     }
             }
@@ -2282,16 +2546,20 @@ void interp( Tensor4View           ia,
   for ( Index ib=0; ib<nb; ++ib )
     {
       const GridPosPoly& tb = bgp[ib];
+      CACHEIDX(b);
       for ( Index ip=0; ip<np; ++ip )
         {
           const GridPosPoly& tp = pgp[ip];
+          CACHEIDX(p);
           for ( Index ir=0; ir<nr; ++ir )
             {
               const GridPosPoly& tr = rgp[ir];
+              CACHEIDX(r);
               for ( Index ic=0; ic<nc; ++ic )
                 {
                   // Current grid position:
                   const GridPosPoly& tc = cgp[ic];
+                  CACHEIDX(c);
 
                   // Get handle to current element of output tensor and
                   // initialize it to zero:
@@ -2304,11 +2572,11 @@ void interp( Tensor4View           ia,
                   LOOPIDX(r)
                   LOOPIDX(c)
                           {
-                            tia += a(*b,
-                                     *p,
-                                     *r,
-                                     *c) * itw(ib,ip,ir,ic,
-                                                     iti);
+                            tia += a.get(*b,
+                                         *p,
+                                         *r,
+                                         *c) * itw.get(ib,ip,ir,ic,
+                                                       iti);
                             ++iti;
                           }
                 }
@@ -2377,19 +2645,24 @@ void interp( Tensor5View           ia,
   for ( Index is=0; is<ns; ++is )
     {
       const GridPosPoly& ts = sgp[is];
+      CACHEIDX(s);
       for ( Index ib=0; ib<nb; ++ib )
         {
           const GridPosPoly& tb = bgp[ib];
+          CACHEIDX(b);
           for ( Index ip=0; ip<np; ++ip )
             {
               const GridPosPoly& tp = pgp[ip];
+              CACHEIDX(p);
               for ( Index ir=0; ir<nr; ++ir )
                 {
                   const GridPosPoly& tr = rgp[ir];
+                  CACHEIDX(r);
                   for ( Index ic=0; ic<nc; ++ic )
                     {
                       // Current grid position:
                       const GridPosPoly& tc = cgp[ic];
+                      CACHEIDX(c);
 
                       // Get handle to current element of output tensor and
                       // initialize it to zero:
@@ -2403,12 +2676,12 @@ void interp( Tensor5View           ia,
                       LOOPIDX(r)
                       LOOPIDX(c)
                                 {
-                                  tia += a(*s,
-                                           *b,
-                                           *p,
-                                           *r,
-                                           *c) * itw(is,ib,ip,ir,ic,
-                                                           iti);
+                                  tia += a.get(*s,
+                                               *b,
+                                               *p,
+                                               *r,
+                                               *c) * itw.get(is,ib,ip,ir,ic,
+                                                             iti);
                                   ++iti;
                                 }
                     }
@@ -2482,22 +2755,28 @@ void interp( Tensor6View           ia,
   for ( Index iv=0; iv<nv; ++iv )
     {
       const GridPosPoly& tv = vgp[iv];
+      CACHEIDX(v);
       for ( Index is=0; is<ns; ++is )
         {
           const GridPosPoly& ts = sgp[is];
+          CACHEIDX(s);
           for ( Index ib=0; ib<nb; ++ib )
             {
               const GridPosPoly& tb = bgp[ib];
+              CACHEIDX(b);
               for ( Index ip=0; ip<np; ++ip )
                 {
                   const GridPosPoly& tp = pgp[ip];
+                  CACHEIDX(p);
                   for ( Index ir=0; ir<nr; ++ir )
                     {
                       const GridPosPoly& tr = rgp[ir];
+                      CACHEIDX(r);
                       for ( Index ic=0; ic<nc; ++ic )
                         {
                           // Current grid position:
                           const GridPosPoly& tc = cgp[ic];
+                          CACHEIDX(c);
 
                           // Get handle to current element of output tensor and
                           // initialize it to zero:
@@ -2512,13 +2791,13 @@ void interp( Tensor6View           ia,
                           LOOPIDX(r)
                           LOOPIDX(c)
                                       {
-                                        tia += a(*v,
-                                                 *s,
-                                                 *b,
-                                                 *p,
-                                                 *r,
-                                                 *c) * itw(iv,is,ib,ip,ir,ic,
-                                                                 iti);
+                                        tia += a.get(*v,
+                                                     *s,
+                                                     *b,
+                                                     *p,
+                                                     *r,
+                                                     *c) * itw.get(iv,is,ib,ip,ir,ic,
+                                                                   iti);
                                         ++iti;
                                       }
                         }
