@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2008 Stefan Buehler <sbuehler@ltu.se>
+/* Copyright (C) 2000-2012 Stefan Buehler <sbuehler@ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -59,12 +59,14 @@
 #include "workspace_ng.h"
 #include "arts_omp.h"
 #include "docserver.h"
+#include "global_data.h"
 
 /** Remind the user of --help and exit return value 1. */
 void polite_goodby()
 {
   cerr << "Try `arts --help' for help.\n";
   arts_exit ();
+
 }
 
 /**
@@ -130,10 +132,9 @@ void option_methods(const String& methods)
   Workspace workspace;
   workspace.initialize();
   // Make global data visible:
-  extern const Array<MdRecord>  md_data_raw;
+  using global_data::md_data_raw;
   extern const Parameters parameters;
-  //  extern const map<String, Index> MdMap;
-  extern const ArrayOfString wsv_group_names;
+  using global_data::wsv_group_names;
 
   // This is used to count the number of matches to a query, so
   // that `none' can be output if necessary
@@ -349,9 +350,8 @@ void option_methods(const String& methods)
 void option_input(const String& input)
 {
   // Make global data visible:
-  extern const Array<MdRecord>  md_data_raw;
-  //  extern const map<String, Index> MdMap;
-  extern const ArrayOfString wsv_group_names;
+  using global_data::md_data_raw;
+  using global_data::wsv_group_names;
 
   // Ok, so the user has probably specified a workspace variable or
   // workspace variable group.
@@ -545,11 +545,10 @@ void option_input(const String& input)
 void option_workspacevariables(const String& workspacevariables)
 {
   // Make global data visible:
-  extern const Array<MdRecord>  md_data;
-  extern const map<String, Index> MdMap;
+  using global_data::md_data;
+  using global_data::MdMap;
   extern const Parameters parameters;
-  //  extern const map<String, Index> WsvMap;
-  extern const ArrayOfString wsv_group_names;
+  using global_data::wsv_group_names;
 
   // This is used to count the number of matches to a query, so
   // that `none' can be output if necessary
@@ -639,9 +638,8 @@ void option_workspacevariables(const String& workspacevariables)
 void option_describe(const String& describe)
 {
   // Make global data visible:
-  extern const Array<MdRecord>  md_data_raw;
-  extern const map<String, Index> MdRawMap;
-  //  extern const ArrayOfString wsv_group_names;
+  using global_data::md_data_raw;
+  using global_data::MdRawMap;
 
   // Let's first assume it is a method that the user wants to have
   // described.
@@ -687,7 +685,7 @@ String arts_mod_time (String filename)
   if (stat (filename.c_str(), &buf) != -1)
     {
       String ts = ctime (&buf.st_mtime);
-      return String(" (") + ts.substr (0, ts.length()-1) + ")";
+      return String(" (compiled ") + ts.substr (0, ts.length()-1) + ")";
     }
   else
       return String("");
@@ -716,11 +714,6 @@ String arts_mod_time (String)
     \author    Stefan Buehler */
 int main (int argc, char **argv)
 {
-
-  // Enable nested parallelism:
-  arts_omp_set_nested(1);
-
-
   extern const Parameters parameters; // Global variable that holds
                                       // all command line parameters. 
 
@@ -785,11 +778,29 @@ int main (int argc, char **argv)
 #else
         << "disabled" << endl
 #endif
-        << "   Disort algorithm:     "
+        << "   Fortran support:      "
+#ifdef FORTRAN_COMPILER
+        << "enabled (" << FORTRAN_COMPILER << ")" << endl
+#else
+        << "disabled" << endl
+#endif
+        << "   Disort support:       "
 #ifdef ENABLE_DISORT
         << "enabled" << endl
 #else
         << "disabled" << endl
+#endif
+        << "   Refice support:       "
+#ifdef ENABLE_REFICE
+        << "enabled" << endl
+#else
+        << "disabled" << endl
+#endif
+        << "   Tmatrix support:      "
+#ifdef ENABLE_TMATRIX
+        << "enabled" << endl
+#else
+        << "disabled" << endl
 #endif
         << ""
         << "Include search paths: " << endl;
@@ -799,6 +810,13 @@ int main (int argc, char **argv)
       {
         osfeatures <<  "   " << (*it) << endl;
       }
+        osfeatures << "Data search paths: " << endl;
+      
+      for (ArrayOfString::const_iterator it = parameters.datapath.begin();
+           it != parameters.datapath.end(); it++)
+      {
+        osfeatures <<  "   " << (*it) << endl;
+      }
     }
 
   if (parameters.version)
@@ -864,10 +882,8 @@ int main (int argc, char **argv)
   define_lineshape_data();
   define_lineshape_norm_data();
 
-  // Make all these data visible:
-  //  extern const Array<MdRecord>  md_data;
-  //  extern const map<String, Index> MdMap;
-  extern const ArrayOfString wsv_group_names;
+  // Make all global data visible:
+  using global_data::wsv_group_names;
 
   // Now we are set to deal with the more interesting command line
   // switches. 
@@ -984,7 +1000,9 @@ int main (int argc, char **argv)
   if ( "" == parameters.basename )
     {
       extern String out_basename;
-      out_basename = parameters.controlfiles[0];
+      ArrayOfString fileparts;
+      parameters.controlfiles[0].split(fileparts, "/");
+      out_basename = fileparts[fileparts.nelem()-1];
       // Find the last . in the name
       String::size_type p = out_basename.rfind(".arts");
 
@@ -1018,7 +1036,7 @@ int main (int argc, char **argv)
   verbosity = verbosity_at_launch;
   verbosity.set_main_agenda(true);
   
-  CREATE_OUTS
+  CREATE_OUTS;
 
   //--------------------< Open report file >--------------------
   // This one needs its own little try block, because we have to
@@ -1039,21 +1057,22 @@ int main (int argc, char **argv)
            << "I have to be able to write to my report file.\n";
       arts_exit ();
     }
-  catch (ios_base::failure x)
-    {
-      cerr << x.what() << "\n"
-           << "I have to be able to write to my report file.\n"
-           << "Make sure you have write permissions for the directory where\n"
-           << "the report file is written.\n";
-      arts_exit ();
-    }
 
   // Now comes the global try block. Exceptions caught after this
   // one are general stuff like file opening errors.
   try
     {
+      out1 << "Executing ARTS.\n";
+      
+      // Output command line:
+      out1 << "Command line:\n";
+      for (Index i=0; i<argc; ++i) {
+          out1 << argv[i] << " ";
+      }
+      out1 << "\n";
+      
       // Output full program name (with version number):		
-      out1 << ARTS_FULL_VERSION << arts_mod_time (argv[0]) << "\n";
+      out1 << "Version: " << ARTS_FULL_VERSION << arts_mod_time (argv[0]) << "\n";
 
       // Output more details about the compilation:
       out2 << osfeatures.str() << "\n";
@@ -1099,32 +1118,41 @@ int main (int argc, char **argv)
       out3 << "\nReading control files:\n";
       for ( Index i=0; i<parameters.controlfiles.nelem(); ++i )
         {
-          out3 << "- " << parameters.controlfiles[i] << "\n";
-
-          // The list of methods to execute and their keyword data from
-          // the control file. 
-          Agenda tasklist;
-
-          Workspace workspace;
-
-          // Call the parser to parse the control text:
-          ArtsParser arts_parser(tasklist, parameters.controlfiles[i], verbosity);
-
-          arts_parser.parse_tasklist();
-
-          tasklist.set_name("Arts");
-
-          tasklist.set_main_agenda();
-
-          //tasklist.find_unused_variables();
-
-          workspace.initialize ();
-
-          // Execute main agenda:
-          Arts2(workspace, tasklist, verbosity);
+          try {
+            out3 << "- " << parameters.controlfiles[i] << "\n";
+            
+            // The list of methods to execute and their keyword data from
+            // the control file. 
+            Agenda tasklist;
+            
+            Workspace workspace;
+            
+            // Call the parser to parse the control text:
+            ArtsParser arts_parser(tasklist, parameters.controlfiles[i], verbosity);
+            
+            arts_parser.parse_tasklist();
+            
+            tasklist.set_name("Arts");
+            
+            tasklist.set_main_agenda();
+            
+            //tasklist.find_unused_variables();
+            
+            workspace.initialize ();
+            
+            // Execute main agenda:
+            Arts2(workspace, tasklist, verbosity);
+          }
+          catch (const runtime_error x)
+          {
+            ostringstream os;
+            os << "Run-time error in controlfile: " << parameters.controlfiles[i] << '\n'
+            << x.what();
+            throw runtime_error(os.str());
+          }
         }
     }
-  catch (runtime_error x)
+  catch (const runtime_error x)
     {
 #ifdef TIME_SUPPORT
       struct tms arts_cputime_end;
