@@ -1,5 +1,5 @@
-/* Copyright (C) 2002-2008
-   Patrick Eriksson <Patrick.Eriksson@rss.chalmers.se>
+/* Copyright (C) 2002-2012
+   Patrick Eriksson <Patrick.Eriksson@chalmers.se>
    Stefan Buehler   <sbuehler@ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
@@ -25,7 +25,7 @@
 
 /*!
    \file   math_funcs.cc
-   \author Patrick Eriksson <Patrick.Eriksson@rss.chalmers.se>
+   \author Patrick Eriksson <Patrick.Eriksson@chalmers.se>
    \date   2000-09-18 
 
    Contains basic mathematical functions.
@@ -549,7 +549,7 @@ Numeric lgamma_func(Numeric xx)
     y=x=xx;
     tmp = x+5.5;
     tmp -= (x+0.5)*log(tmp);
-    ser = 1.000000000190015;    // FIXME OLE: ser will be 1 if Numeric==float
+    ser = 1.000000000190015;
     for (Index j=0;j<=5;j++) ser += cof[j]/++y;
     return -tmp+log(2.5066282746310005*ser/x);
   }
@@ -561,4 +561,26 @@ Numeric lgamma_func(Numeric xx)
     throw runtime_error(os.str());
   }
 }
+
+
+
+//! lunit
+/*!
+    Normalises a vector to have unit length.
+
+    The standard Euclidean norm is used (2-norm).
+
+    param    x   In/Out: A vector.
+
+    \author Patrick Eriksson
+    \date   2012-02-12
+*/
+void unitl( Vector& x )
+{
+  assert( x.nelem() > 0 );
+ 
+  const Numeric l = sqrt(x*x);
+  for(Index i=0; i<x.nelem(); i++ )
+    x[i] /= l;
+}
         
