@@ -1,5 +1,5 @@
-/* Copyright (C) 2002-2008
-   Patrick Eriksson <Patrick.Eriksson@rss.chalmers.se>
+/* Copyright (C) 2002-2012
+   Patrick Eriksson <Patrick.Eriksson@chalmers.se>
    Stefan Buehler   <sbuehler@ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
@@ -25,7 +25,7 @@
 
 /*!
    \file   math_funcs.h
-   \author Patrick Eriksson <Patrick.Eriksson@rss.chalmers.se>
+   \author Patrick Eriksson <Patrick.Eriksson@chalmers.se>
    \date 2000-09-18 
 
    Contains the decleration of the functions in math_funcs.cc.
@@ -38,8 +38,6 @@
 
 #include "matpackI.h"
 
-
-
 Numeric fac(const Index n);
 
 Index integer_div( const Index& x, const Index& y );
@@ -84,4 +82,6 @@ Numeric gamma_func(Numeric xx);
 
 Numeric lgamma_func(Numeric xx);
 
+void unitl( Vector& x );
+
 #endif  // math_funcs_h
