@@ -1,4 +1,4 @@
-/* Copyright (C) 2002-2008 Stefan Buehler <sbuehler@ltu.se>
+/* Copyright (C) 2002-2012 Stefan Buehler <sbuehler@ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -25,6 +25,9 @@
 #include "matpackIV.h"
 #include "exceptions.h"
 
+using std::runtime_error;
+
+
 /** The -> operator is needed, so that we can write i->begin() to get
     the 3D iterators. */
 Tensor3View* Iterator4D::operator->()
@@ -381,13 +384,13 @@ ConstVectorView ConstTensor4View::operator()(Index b,
 */
 Numeric *Tensor4View::get_c_array()
 {
-  if (mbr.mstart != 0 || mbr.mstride != mpr.mextent * mrr.mextent * mcr.mextent
-      || mpr.mstart != 0 || mpr.mstride != mrr.mextent * mcr.mextent
-      || mrr.mstart != 0 || mrr.mstride != mcr.mextent
-      || mcr.mstart != 0 || mcr.mstride != 1)
-    throw runtime_error("A Tensor4View can only be converted to a plain C-array if mbr.mstart == 0 and mbr.mstride == mrp.extent*mrr.extent*mcr.extent and mpr.mstart == 0 and mpr.mstride == mrr.extent*mcr.extent and mrr.mstart == 0 and mrr.mstride == mcr.extent and mcr.mstart == 0 and mcr.mstride == 1");
+    if (mbr.mstart != 0 || mbr.mstride != mpr.mextent * mrr.mextent * mcr.mextent
+        || mpr.mstart != 0 || mpr.mstride != mrr.mextent * mcr.mextent
+        || mrr.mstart != 0 || mrr.mstride != mcr.mextent
+        || mcr.mstart != 0 || mcr.mstride != 1)
+        throw std::runtime_error("A Tensor4View can only be converted to a plain C-array if it's pointing to a continuous block of data");
 
-  return mdata;
+    return mdata;
 }
 
 /** Conversion to plain C-array.
@@ -402,7 +405,7 @@ const Numeric *Tensor4View::get_c_array() const
       || mpr.mstart != 0 || mpr.mstride != mrr.mextent * mcr.mextent
       || mrr.mstart != 0 || mrr.mstride != mcr.mextent
       || mcr.mstart != 0 || mcr.mstride != 1)
-    throw runtime_error("A Tensor4View can only be converted to a plain C-array if mbr.mstart == 0 and mbr.mstride == mrp.extent*mrr.extent*mcr.extent and mpr.mstart == 0 and mpr.mstride == mrr.extent*mcr.extent and mrr.mstart == 0 and mrr.mstride == mcr.extent and mcr.mstart == 0 and mcr.mstride == 1");
+      throw std::runtime_error("A Tensor4View can only be converted to a plain C-array if it's pointing to a continuous block of data");
 
   return mdata;
 }
@@ -493,7 +496,7 @@ ConstTensor4View::ConstTensor4View(Numeric *data,
 /** Output operator. This demonstrates how iterators can be used to
     traverse the tensor. We use the standard output operator for
     Tensor to print each book in turn. */
-ostream& operator<<(ostream& os, const ConstTensor4View& v)
+std::ostream& operator<<(std::ostream& os, const ConstTensor4View& v)
 {
   // Page iterators:
   ConstIterator4D ib = v.begin();
@@ -1380,13 +1383,9 @@ Tensor4::Tensor4(const Tensor4& m) :
   \author Stefan Buehler
   \date   2002-12-19
 */
-Tensor4& Tensor4::operator=(const Tensor4& m)
+Tensor4& Tensor4::operator=(Tensor4 x)
 {
-  //  cout << "Tensor4 copy: m = " << m.nrows() << " " << m.ncols() << "\n";
-  //  cout << "              n = " << nrows() << " " << ncols() << "\n";
-
-  resize( m.mbr.mextent, m.mpr.mextent, m.mrr.mextent, m.mcr.mextent );
-  copy( m.begin(), m.end(), begin() );
+  swap(*this, x);
   return *this;
 }
 
@@ -1434,6 +1433,18 @@ void Tensor4::resize(Index b, Index p, Index r, Index c)
     }
 }
 
+
+/** Swaps two objects. */
+void swap(Tensor4& t1, Tensor4& t2)
+{
+  std::swap(t1.mbr, t2.mbr);
+  std::swap(t1.mpr, t2.mpr);
+  std::swap(t1.mrr, t2.mrr);
+  std::swap(t1.mcr, t2.mcr);
+  std::swap(t1.mdata, t2.mdata);
+}
+
+
 /** Destructor for Tensor4. This is important, since Tensor4 uses new to
     allocate storage. */
 Tensor4::~Tensor4()
