@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2008 Stefan Buehler <sbuehler@ltu.se>
+/* Copyright (C) 2000-2012 Stefan Buehler <sbuehler@ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -34,6 +34,22 @@
 #include "wsv_aux.h"
 #include "workspace_ng.h"
 
+namespace global_data {
+//! The map associated with md_data.
+map<String, Index> MdMap;
+//! The map associated with md_data_raw.
+map<String, Index> MdRawMap;
+//! Lookup information for workspace methods.
+/*!
+ This is the data with expanded supergeneric methods. That means,
+ e.g., instead of supergeneric method Copy(Any,Any) there will be
+ Copy(Vector,Vector), Copy(Matrix,Matrix), etc..
+ */
+Array<MdRecord> md_data;
+
+extern const Array<MdRecord> md_data_raw;
+extern const ArrayOfString wsv_group_names;
+}
 
 void limit_line_length( ostream& os, ostringstream& curline,
                         ostringstream& token, const String& indent,
@@ -60,7 +76,7 @@ MdRecord::MdRecord(const char                   name[],
                    const MakeArray<String>&     gindesc,
                    bool                         set_method,
                    bool                         agenda_method,
-                   bool                         suppress_header,
+                   bool                         uses_templates,
                    bool                         pass_workspace,
                    bool                         pass_wsv_names ) :
     mname(            name                ),
@@ -78,7 +94,7 @@ MdRecord::MdRecord(const char                   name[],
     mset_method(      set_method          ),
     magenda_method(   agenda_method       ),
     msupergeneric(    false               ),
-    msuppress_header( suppress_header     ),
+    muses_templates(  uses_templates      ),
     mpass_workspace(  pass_workspace      ),
     mpass_wsv_names(  pass_wsv_names      ),
     mactual_groups(   ""                  )
@@ -134,6 +150,14 @@ MdRecord::MdRecord(const char                   name[],
         {
           mgouttype[j] = types[0];
           mgoutspectype[j].resize(0);
+          if (types[0] == get_wsv_group_id("Any") && !muses_templates)
+          {
+              ostringstream os;
+              os << "WSM " << mname << " takes \"Any\" as input and\n"
+              << "therefore must be implemented as a template function.\n"
+              << "Pass USES_TEMPLATES(true) in methods.cc!";
+              throw runtime_error(os.str());
+          }
         }
       else if (types.nelem() > 1)
         {
@@ -161,6 +185,14 @@ MdRecord::MdRecord(const char                   name[],
         {
           mgintype[j] = get_wsv_group_id (gintype[j]);
           mginspectype[j].resize(0);
+          if (types[0] == get_wsv_group_id("Any") && !muses_templates)
+          {
+              ostringstream os;
+              os << "WSM " << mname << " defines \"Any\" as output and\n"
+              << "therefore must be implemented as a template function.\n"
+              << "Pass USES_TEMPLATES(true) in methods.cc!";
+              throw runtime_error(os.str());
+          }
         }
       else if (types.nelem() > 1)
         {
@@ -269,7 +301,7 @@ void MdRecord::subst_any_with_group( Index g )
 {
   const Index wsv_group_id_Any = get_wsv_group_id("Any");
   // The group names, we need them for the expansion:
-  extern const ArrayOfString wsv_group_names;
+  using global_data::wsv_group_names;
 
   // Make sure they are initialized:
   assert( 0 != wsv_group_names.nelem() );
@@ -313,7 +345,7 @@ void MdRecord::subst_any_with_group( Index g )
 void MdRecord::subst_any_with_specific_group( Index g )
 {
   // The group names, we need them for the expansion:
-  extern const ArrayOfString wsv_group_names;
+  using global_data::wsv_group_names;
 
   const Index wsv_group_id_Any = get_wsv_group_id("Any");
 
@@ -363,11 +395,11 @@ void MdRecord::subst_any_with_specific_group( Index g )
 */
 void expand_md_data_raw_to_md_data()
 {
-  extern const Array<MdRecord> md_data_raw;
-  extern Array<MdRecord>       md_data;
+  using global_data::md_data_raw;
+  using global_data::md_data;
 
   // The group names, we need them for the expansion:
-  extern const ArrayOfString wsv_group_names;
+  using global_data::wsv_group_names;
 
   const Index wsv_group_id_Any = get_wsv_group_id("Any");
 
@@ -440,9 +472,9 @@ void expand_md_data_raw_to_md_data()
 void define_md_map()
 {
   // md_data is constant here and should never be changed
-  extern Array<MdRecord> md_data;
-  extern map<String, Index> MdMap;
-  DEBUG_ONLY( extern const ArrayOfString wsv_group_names; )
+  using global_data::md_data;
+  using global_data::MdMap;
+  DEBUG_ONLY( using global_data::wsv_group_names; )
 
   // Check that md_data and wsv_group_names have already be defined:
   assert( 0 != md_data.nelem() );
@@ -481,8 +513,8 @@ void define_md_map()
 */
 void define_md_raw_map()
 {
-  extern const Array<MdRecord> md_data_raw;
-  extern map<String, Index> MdRawMap;
+  using global_data::md_data_raw;
+  using global_data::MdRawMap;
 
   for ( Index i=0 ; i<md_data_raw.nelem() ; ++i)
     {
@@ -556,7 +588,7 @@ void get_short_wsv_description(String &s, const String &desc)
 ostream& MdRecord::PrintTemplate(ostream& os,
                                  bool show_description) const
 {
-  extern const  ArrayOfString wsv_group_names;
+  using global_data::wsv_group_names;
 
   if (show_description)
     {
@@ -640,7 +672,7 @@ void limit_line_length( ostream& os,
 //! Output operator for MdRecord.
 ostream& operator<<(ostream& os, const MdRecord& mdr)
 {
-  extern const ArrayOfString wsv_group_names;
+  using global_data::wsv_group_names;
   bool first;
   ostringstream buf;
   ostringstream param;
