@@ -1,4 +1,4 @@
-/* Copyright (C) 2003-2008 Claudia Emde <claudia.emde@dlr.de>
+/* Copyright (C) 2003-2012 Claudia Emde <claudia.emde@dlr.de>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -75,7 +75,7 @@ extern const Numeric PI;
   \param abs_vec_data Absorption vector in database.
   \param za_datagrid Zenith angle grid in the database.
   \param aa_datagrid Zenith angle grid in the database.
-  \param ptype Clasiification of the hydometeor species.
+  \param particle_type Clasiification of the hydometeor species.
   \param za_sca Zenith angle of scattered direction.
   \param aa_sca Azimuth angle of scattered direction.
      
@@ -88,7 +88,7 @@ void abs_vecTransform(//Output and Input
                       ConstTensor3View abs_vec_data,
                       ConstVectorView za_datagrid,
                       ConstVectorView aa_datagrid _U_,
-                      const ParticleType& ptype,
+                      const ParticleType& particle_type,
                       const Numeric& za_sca _U_,
                       const Numeric& aa_sca _U_,
                       const Verbosity& verbosity)
@@ -100,11 +100,11 @@ void abs_vecTransform(//Output and Input
                         "must be 1,2,3 or 4");
   }
   
-  switch (ptype){
+  switch (particle_type){
       
     case PARTICLE_TYPE_GENERAL:
     {
-      CREATE_OUT0
+      CREATE_OUT0;
       out0 << "Case PARTICLE_TYPE_GENERAL not yet implemented. \n"; 
       break;
     }
@@ -158,7 +158,7 @@ void abs_vecTransform(//Output and Input
     }
     default:
     {
-      CREATE_OUT0
+      CREATE_OUT0;
       out0 << "Not all particle type cases are implemented\n";
     }
   }  
@@ -180,7 +180,7 @@ void abs_vecTransform(//Output and Input
   \param ext_mat_data Extinction matrix in database.
   \param za_datagrid Zenith angle grid in the database.
   \param aa_datagrid Zenith angle grid in the database.
-  \param ptype Clasiification of the hydometeor species.
+  \param particle_type Clasiification of the hydometeor species.
   \param za_sca Zenith angle of scattered direction.
   \param aa_sca Azimuth angle of scattered direction.
 
@@ -193,23 +193,23 @@ void ext_matTransform(//Output and Input
                       ConstTensor3View ext_mat_data,
                       ConstVectorView za_datagrid,
                       ConstVectorView aa_datagrid _U_,
-                      const ParticleType& ptype,
+                      const ParticleType& particle_type,
                       const Numeric& za_sca,
                       const Numeric& aa_sca _U_,
                       const Verbosity& verbosity)
 {
   const Index stokes_dim = ext_mat_lab.ncols();
-  
+
   if (stokes_dim > 4 || stokes_dim < 1){
     throw runtime_error("The dimension of the stokes vector \n"
                         "must be 1,2,3 or 4");
   }
   
-  switch (ptype){
+  switch (particle_type){
       
     case PARTICLE_TYPE_GENERAL:
     {
-      CREATE_OUT0
+      CREATE_OUT0;
       out0 << "Case PARTICLE_TYPE_GENERAL not yet implemented. \n"; 
       break;
     }
@@ -311,7 +311,7 @@ void ext_matTransform(//Output and Input
     }
     default:
     {
-      CREATE_OUT0
+      CREATE_OUT0;
       out0 << "Not all particle type cases are implemented\n";
     }
   }
@@ -331,7 +331,7 @@ void ext_matTransform(//Output and Input
   \param[in]     pha_mat_data  Phase matrix in database.
   \param[in]     za_datagrid   Zenith angle grid in the database.
   \param[in]     aa_datagrid   Zenith angle grid in the database.
-  \param[in]     ptype         Classification of the hydometeor species.
+  \param[in]     particle_type Classification of the hydometeor species.
   \param[in]     za_sca_idx    Index of zenith angle of scattered direction.
   \param[in]     aa_sca_idx    Index of azimuth angle of scattered direction.
   \param[in]     za_inc_idx    Index of zenith angle of incoming direction.
@@ -348,7 +348,7 @@ void pha_matTransform(//Output
                       ConstTensor5View pha_mat_data,
                       ConstVectorView za_datagrid,
                       ConstVectorView aa_datagrid,
-                      const ParticleType& ptype,
+                      const ParticleType& particle_type,
                       const Index& za_sca_idx,
                       const Index& aa_sca_idx,
                       const Index& za_inc_idx,
@@ -369,11 +369,11 @@ void pha_matTransform(//Output
                         "must be 1,2,3 or 4");
   }
   
-  switch (ptype){
+  switch (particle_type){
       
     case PARTICLE_TYPE_GENERAL:
     {
-      CREATE_OUT0
+      CREATE_OUT0;
       out0 << "Case PARTICLE_TYPE_GENERAL not yet implemented. \n"; 
       break;
     }
@@ -528,7 +528,7 @@ void pha_matTransform(//Output
     }  
     default:
     {
-      CREATE_OUT0
+      CREATE_OUT0;
       out0 << "Not all particle type cases are implemented\n";
     }
   }
@@ -536,6 +536,60 @@ void pha_matTransform(//Output
 
 
 
+//! Derive extinction matrix from absorption vector.
+/*! 
+  In case, when only absorption of a particle shall be considered, and the
+  scattering is negelected, the extinction matrix is set from the absorption
+  vector only.
+
+  Extinction matrix is set the following way:
+  
+  K11 = K22 = K33 = K44 = a1
+  K12 = K21 = a2
+  K13 = K31 = a3
+  K14 = K41 = a4
+
+  Other elements remain 0.
+  However, note that the other elements might be supposed to contain non-zero
+  values as well. We couldn't find an appropriate solution yet, though, and
+  these elements are expected to be rather small. Also, using the absorption
+  part only is anyway only a rough approximation. Hence, we deem the assumption
+  of setting the remaining elements to zero reasonable.
+
+  Output and Input:
+  \param ext_mat     Extinction matrix.
+  Input: 
+  \param abs_vec     Absorption vector.
+  \param stokes_dim  as the WSV.
+  
+  \author Jana Mendrok
+  \date   2013-04-30 
+*/
+void ext_matFromabs_vec(//Output:
+                        MatrixView ext_mat,
+                        //Input:
+                        ConstVectorView abs_vec,
+                        const Index& stokes_dim)
+{
+  assert( stokes_dim>=1  &&  stokes_dim<=4 );
+  assert( ext_mat.nrows() == stokes_dim );
+  assert( ext_mat.ncols() == stokes_dim );
+  assert( abs_vec.nelem() == stokes_dim );
+
+  // first: diagonal elements
+  for (Index is=0; is<stokes_dim; is++)
+    {
+      ext_mat(is,is) += abs_vec[0];
+    }
+  // second: off-diagonal elements, namely first row and column
+  for (Index is=1; is<stokes_dim; is++)
+    {
+      ext_mat(0,is) += abs_vec[is];
+      ext_mat(is,0) += abs_vec[is];
+    }
+}
+
+
 
 //! Interpolate data on the scattering angle.
 /*! 
@@ -912,3 +966,176 @@ ostream& operator<< (ostream& os, const ArrayOfScatteringMetaData& /*assd*/)
   os << "ArrayOfScatteringMetaData: Output operator not implemented";
   return os;
 }
+
+
+//! Get optical properties from propmat_clearsky
+/*!
+  This turns propmat_clearsky into the extinction matrix
+  and absorption vector for use when these are important.
+
+  Internal function to replace the old opt_prop_gas_agenda.
+  
+  Output and Input:
+  \param ext_mat Extinction matrix.
+  \param abs_vec Absorption vector.
+  Input:
+  \param propmat_clearsky as the WSV.
+
+  \author Richard Larsson
+  \date   2012-07-24
+*/
+void opt_prop_sum_propmat_clearsky(//Output:
+                                      Tensor3&         ext_mat,
+                                      Matrix&          abs_vec,
+                                      //Input:
+                                      const Tensor4    propmat_clearsky)
+{
+
+    Index stokes_dim = propmat_clearsky.ncols();
+    
+    Index freq_dim = propmat_clearsky.npages();
+
+    // old abs_vecInit
+    abs_vec.resize( freq_dim, stokes_dim );
+    abs_vec = 0;                  // Initialize to zero!
+
+    // old ext_matInit
+    ext_mat.resize( freq_dim, stokes_dim, stokes_dim );
+    ext_mat = 0;                  // Initialize to zero!
+
+   // old ext_matAddGas and abs_vecAddGas for 0 vector and matrix
+    for ( Index iv=0; iv<freq_dim; ++iv )
+        for ( Index is1=0; is1<stokes_dim; ++is1 )
+        {
+            abs_vec(iv,is1) += propmat_clearsky(joker,iv,is1,0).sum();
+            for ( Index is2=0; is2<stokes_dim; ++is2 )
+                ext_mat(iv,is1,is2) += propmat_clearsky(joker,iv,is1,is2).sum();
+        }
+}
+
+
+//! Convert particle name to enum value
+/*!
+ Returns the ParticleType enum value for the given String.
+
+ \param[in]  particle_type_string  Particle type name
+ \return     ParticleType enum value
+
+ \author Oliver Lemke
+ */
+ParticleType ParticleTypeFromString(const String& particle_type_string)
+{
+    ParticleType particle_type;
+    if (particle_type_string == "general")
+        particle_type = PARTICLE_TYPE_GENERAL;
+    else if (particle_type_string == "macroscopically_isotropic")
+        particle_type = PARTICLE_TYPE_MACROS_ISO;
+    else if (particle_type_string == "horizontally_aligned")
+        particle_type = PARTICLE_TYPE_HORIZ_AL;
+    else if (particle_type_string == "spherical")
+        particle_type = PARTICLE_TYPE_SPHERICAL;
+    else
+    {
+        ostringstream os;
+        os << "Unknown particle type: " << particle_type_string << endl
+           << "Valid types are: general, macroscopically_isotropic, "
+           << "horizontally_aligned and spherical.";
+        throw std::runtime_error(os.str());
+    }
+
+    return particle_type;
+}
+
+
+//! Convert particle type enum value to String.
+/*!
+ Returns the ParticleType enum value for the given String.
+
+ \param[in]  particle_type  Particle type
+ \return     String representation of ParticleType
+
+ \author Oliver Lemke
+ */
+String ParticleTypeToString(const ParticleType& particle_type)
+{
+    String particle_type_string;
+
+    switch (particle_type) {
+        case PARTICLE_TYPE_GENERAL:
+            particle_type_string = "general";
+            break;
+        case PARTICLE_TYPE_MACROS_ISO:
+            particle_type_string = "macroscopically_isotropic";
+            break;
+        case PARTICLE_TYPE_HORIZ_AL:
+            particle_type_string = "horizontally_aligned";
+            break;
+        case PARTICLE_TYPE_SPHERICAL:
+            particle_type_string = "spherical";
+            break;
+        default:
+            ostringstream os;
+            os << "Internal error: Cannot map ParticleType enum value "
+            << particle_type << " to String.";
+            throw std::runtime_error(os.str());
+            break;
+    }
+
+    return particle_type_string;
+}
+
+
+//! Convert particle ssd method name to enum value
+/*!
+ Returns the ParticleSSDMethod enum value for the given String.
+
+ \param[in]  particle_ssdmethod_string  Particle SSD method name
+ \return     ParticleSSDMethod enum value
+
+ \author Oliver Lemke
+ */
+ParticleSSDMethod ParticleSSDMethodFromString(const String& particle_ssdmethod_string)
+{
+    ParticleSSDMethod particle_ssdmethod;
+    if (particle_ssdmethod_string == "tmatrix")
+        particle_ssdmethod = PARTICLE_SSDMETHOD_TMATRIX;
+    else
+    {
+        ostringstream os;
+        os << "Unknown particle SSD method: " << particle_ssdmethod_string << endl
+        << "Valid methods: tmatrix";
+        throw std::runtime_error(os.str());
+    }
+
+    return particle_ssdmethod;
+}
+
+
+//! Convert particle type enum value to String.
+/*!
+ Returns the ParticleType enum value for the given String.
+
+ \param[in]  particle_type  Particle type
+ \return     String representation of ParticleSSDMethod
+
+ \author Oliver Lemke
+ */
+String ParticleTypeToString(const ParticleSSDMethod& particle_ssdmethod)
+{
+    String particle_ssdmethod_string;
+
+    switch (particle_ssdmethod) {
+        case PARTICLE_SSDMETHOD_TMATRIX:
+            particle_ssdmethod_string = "tmatrix";
+            break;
+        default:
+            ostringstream os;
+            os << "Internal error: Cannot map ParticleSSDMethod enum value "
+            << particle_ssdmethod << " to String.";
+            throw std::runtime_error(os.str());
+            break;
+    }
+
+    return particle_ssdmethod_string;
+}
+
