@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2008 Stefan Buehler <sbuehler@ltu.se>
+/* Copyright (C) 2000-2012 Stefan Buehler <sbuehler@ltu.se>
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -49,11 +49,13 @@ public:
     help(false),
     version(false),
     basename(""),
+    outdir(""),
     controlfiles(),
     reporting(-1),
     methods(""),
     numthreads(0),
     includepath(),
+    datapath(),
     input(""),
     workspacevariables(""),
     describe(""),
@@ -64,6 +66,7 @@ public:
     daemon(false),
     gui(false)
   { /* Nothing to be done here */ }
+  
   /** Short message how to call the program. */
   String usage;
   /** Longer message explaining the options. */
@@ -76,6 +79,11 @@ public:
       as the base name for the report file and for other output
       files. */ 
   String basename;
+  /** If this is specified (with the -o --outdir option), it is used
+   as the base directory for the report file and for other output
+   files. If a full path is given for an output file it will not
+   be affected by this. */ 
+  String outdir;
   /** The filenames of the controlfiles. Can be only one or as many as
       you want. */
   ArrayOfString controlfiles;
@@ -95,6 +103,8 @@ public:
   Index numthreads;
   /** List of paths to search for include files. */
   ArrayOfString includepath; 
+  /** List of paths to search for data files. */
+  ArrayOfString datapath; 
   /** This is complementary to the methods switch. It must be given
       the name of a variable (or group). Then it lists all methods that take this
       variable (or group) as input. */
